/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package RodStacker;

import java.applet.Applet;
import java.awt.*;
import hamlet.*;
import RodStacker.*;

class TrigPanel extends Panel
{
private point p[];
private point origP[];
private Graphics offGraphics = null;
private Image offImage = null;
boolean showA = false;

private final static int FRAME_WIDTH = 300;
private final static int FRAME_HEIGHT = 300;
private final static int TOP_MARGIN = 5;
private final static int BOTTOM_MARGIN = 5;
private final static int LEFT_MARGIN = 20;
private final static int RIGHT_MARGIN = 25;
private final static int NUM_POINTS = 3;
  
public TrigPanel()
  {
    p = new point[NUM_POINTS];
    origP = new point[NUM_POINTS];
    SetPoints(null, null, null);
  }
  
public TrigPanel(point iPTop, point iPLeft, point iPRight)
  {
    p = new point[NUM_POINTS];
    origP = new point[NUM_POINTS];
    SetPoints(iPTop, iPLeft, iPRight);
  }
  
public void reset()
  {
    SetPoints(null, null, null);
    repaint();
  }
  
public void SetPoints(point iPTop, point iPLeft, point iPRight)
  {
    if (iPTop != null) //don't rescale if we don't have any points
    {
	p[0] = new point(iPTop.x, iPTop.y, iPTop.radius, null, null, -1, -1);
	p[1] = new point(iPLeft.x, iPLeft.y, iPLeft.radius, null, null, 
			 -1, -1);
	p[2] = new point(iPRight.x, iPRight.y, iPRight.radius, null, null, 
			 -1, -1);
	
	origP[0] = new point(iPTop.x, iPTop.y, iPTop.radius, null, null, 
			     -1, -1);
	origP[1] = new point(iPLeft.x, iPLeft.y, iPLeft.radius, null, null, 
			     -1, -1);
	origP[2] = new point(iPRight.x, iPRight.y, iPRight.radius, null, null,
			     -1, -1);
	
	Rescale();
    }
    else
    {
		p[0] = null; p[1] = null; p[2] = null;
    }
  }
  
public void Rescale() //rescale the points to fit the screen
  {
    int i;
    int max_x = p[0].x, max_y = p[0].y, min_x = p[0].x, min_y = p[0].y;
    double scale=0.0;
    
    for (i = 0; i < NUM_POINTS; i++)
      {
	if (p[i].x - p[i].radius < min_x)
	  min_x = p[i].x - p[i].radius;
	if (p[i].x + p[i].radius > max_x)
	  max_x = p[i].x + p[i].radius;
	if (p[i].y + p[i].radius > max_y)
	  max_y = p[i].y + p[i].radius;
	if (p[i].y - p[i].radius < min_y)
	  min_y = p[i].y - p[i].radius;
      }
    
//	System.out.println(scale + "  " + max_x + "  " 
//        + min_x + "  " + max_y + "  " + min_y);
    scale = Math.min((double)(size().width - LEFT_MARGIN - RIGHT_MARGIN) / 
		     (max_x - min_x),
		     (double)(size().height - TOP_MARGIN - BOTTOM_MARGIN) / 
		     (max_y - min_y));
//	System.out.println(scale + "  " + max_x + "  " + min_x + 
//     "  " + max_y + "  " + min_y);
    for (i = 0; i < NUM_POINTS; i++)
    {
    	origP[i].x = p[i].x;
    	origP[i].y = p[i].y;
    	origP[i].radius = p[i].radius;
	p[i].x = (int) 
	            Math.round(p[i].x * scale - min_x * scale + LEFT_MARGIN);
	p[i].y = (int) 
	            Math.round(p[i].y * scale - min_y *scale + BOTTOM_MARGIN);
	p[i].radius = (int) Math.round(scale * p[i].radius);
    }
  }
  
public synchronized void reshape(int x, int y, int width, int height)
  {
    super.reshape(x,y,width,height);
    try{
      offImage = createImage(width, height);
      offGraphics = offImage.getGraphics();
//      offGraphics.setFont( new Font("Courier", Font.PLAIN, 9));
    }
    catch (Exception e)
      {}
    SetPoints(null, null, null);
    repaint();
  }
  
public Dimension minimumSize()
  {
    return new Dimension(FRAME_WIDTH, FRAME_HEIGHT);
  }
  
public Dimension preferredSize()
  {
    return new Dimension(FRAME_WIDTH, FRAME_HEIGHT);
  }
  
public void update(Graphics g)
  {
    paint(g);
  }
  
public void paint(Graphics g)
{
  int i, xpos;
  String tempStr;
  
  if (offImage == null)
    {
      offImage = createImage(size().width, size().height);
      offGraphics = offImage.getGraphics();
 //     offGraphics.setFont( new Font("Courier", Font.PLAIN, 9));
    }
  offGraphics.setColor(getBackground());
  offGraphics.fillRect(0, 0, size().width, size().height);
  offGraphics.setColor(Color.black);
  offGraphics.drawRect(0,0,size().width, size().height);
  if (p[0] != null) //don't draw anything if we have no points
    {
      offGraphics.setColor(Color.red);
      for (i = 0; i < NUM_POINTS; i++)
	{
	  if (i > 0)
	    offGraphics.setColor(Color.magenta);
	  offGraphics.drawOval(p[i].x - p[i].radius, 
			       size().height - p[i].y - p[i].radius,
			       p[i].radius*2, 
			       p[i].radius*2);
	}
      offGraphics.setColor(Color.black);
      for (i = 0; i < NUM_POINTS; i++)
	{
	  offGraphics.drawLine(p[i].x, size().height - p[i].y, 
			       p[(i+1) % 3].x, size().height - p[(i+1) % 3].y);
	}
      if (p[1].y > p[2].y) //the left point is higher than the right point
	{
	  offGraphics.drawLine(p[1].x, size().height - p[1].y,
			       p[1].x, size().height - p[2].y);
	  offGraphics.drawLine(p[1].x, size().height - p[2].y,
			       p[2].x, size().height - p[2].y);
	}
      else
	{
	  offGraphics.drawLine(p[1].x, size().height - p[1].y,
			       p[2].x, size().height - p[1].y);
	  offGraphics.drawLine(p[2].x, size().height - p[2].y,
			       p[2].x, size().height - p[1].y);
	}
      //draw the (x,y) positions of the rods' centers and the circles' radii
      //top
      if (showA)
	{
	  tempStr = "(" + Integer.toString(origP[0].x) + "," + 
	    Integer.toString(origP[0].y) + ")";
	  offGraphics.drawString(tempStr, 
				 p[0].x - 20,
				 size().height - p[0].y - 15);
	}
      tempStr = "r= " + Integer.toString(origP[0].radius);
      offGraphics.drawString(tempStr, p[0].x - 20, size().height - p[0].y - 5);
      //left
      tempStr = "(" + Integer.toString(origP[1].x) + "," + 
	Integer.toString(origP[1].y) + ")";
      xpos = p[1].x - g.getFontMetrics().stringWidth(tempStr) - 5;
		offGraphics.drawString(tempStr, xpos, size().height - p[1].y);
      tempStr = "r= " + Integer.toString(origP[1].radius);
      offGraphics.drawString(tempStr, xpos, size().height - p[1].y + 10);
      //right
      tempStr = "(" + Integer.toString(origP[2].x) + "," + 
	Integer.toString(origP[2].y) + ")";
	  xpos = p[2].x + 5;
	  if (xpos + g.getFontMetrics().stringWidth(tempStr) > size().width)
	  	xpos = size().width - 
		          g.getFontMetrics().stringWidth(tempStr) - 5;
      offGraphics.drawString(tempStr, xpos, size().height - p[2].y);
      tempStr = "r= " + Integer.toString(origP[2].radius);
      offGraphics.drawString(tempStr, xpos, size().height - p[2].y + 10);
		
    }
  g.drawImage(offImage, 0, 0, getBackground(), null);
}

public void ShowAnswer()
{
  showA = true;
}
	
public void HideAnswer()
{
  showA = false;
}

}


